package com.jiuqi.util;

import java.util.Collections;
import java.util.ListIterator;
import java.util.Iterator;
import java.util.Collection;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;

public final class SortedList implements List
{
    private List items;
    private Comparator comparator;
    private int[] findStub;
    
    public SortedList() {
        this.findStub = new int[1];
        this.items = new ArrayList();
    }
    
    public SortedList(final int initialCapacity) {
        this.findStub = new int[1];
        this.items = new ArrayList(initialCapacity);
    }
    
    public SortedList(final Comparator comparator) {
        this.findStub = new int[1];
        this.items = new ArrayList();
        this.comparator = comparator;
    }
    
    public SortedList(final int initalCapacity, final Comparator comparator) {
        this.findStub = new int[1];
        this.items = new ArrayList(initalCapacity);
        this.comparator = comparator;
    }
    
    public int compareObjects(final Object o1, final Object o2) {
        if (this.comparator != null) {
            return this.comparator.compare(o1, o2);
        }
        if (o1 instanceof Comparable) {
            return ((Comparable)o1).compareTo(o2);
        }
        if (o2 instanceof Comparable) {
            return -((Comparable)o2).compareTo(o1);
        }
        throw new UnsupportedOperationException("对象比较时无法转化为Comparable接口！");
    }
    
    public boolean add(final Object o) {
        final int[] index = { 0 };
        this.find(o, index);
        this.items.add(index[0], o);
        return true;
    }
    
    public void add(final int index, final Object element) {
        this.add(element);
    }
    
    public void _add(final int index, final Object element) {
        this.items.add(index, element);
    }
    
    public boolean addAll(final Collection c) {
        final List newItems = new ArrayList(c);
        this.reset(newItems);
        return true;
    }
    
    public boolean addAll(final int index, final Collection c) {
        return this.addAll(c);
    }
    
    public void clear() {
        this.items.clear();
    }
    
    public boolean contains(final Object o) {
        return this.find(o, new int[1]);
    }
    
    public boolean containsAll(final Collection c) {
        final Iterator itr = c.iterator();
        while (itr.hasNext()) {
            if (!this.contains(itr.next())) {
                return false;
            }
        }
        return true;
    }
    
    public Object get(final int index) {
        return this.items.get(index);
    }
    
    public int indexOf(final Object o) {
        int low = 0;
        int high = this.size() - 1;
        while (low <= high) {
            final int mid = (low + high) / 2;
            final int flag = this.compareObjects(o, this.get(mid));
            if (flag == 0) {
                if (mid == low) {
                    return mid;
                }
                high = mid;
            }
            else if (flag > 0) {
                low = mid + 1;
            }
            else {
                high = mid - 1;
            }
        }
        return -1;
    }
    
    public boolean isEmpty() {
        return this.items.isEmpty();
    }
    
    public Iterator iterator() {
        return this.items.iterator();
    }
    
    public int lastIndexOf(final Object o) {
        int low = 0;
        int high = this.size() - 1;
        while (low <= high) {
            final int mid = (low + high + 1) / 2;
            final int flag = this.compareObjects(o, this.get(mid));
            if (flag == 0) {
                if (mid == high) {
                    return mid;
                }
                low = mid;
            }
            else if (flag > 0) {
                low = mid + 1;
            }
            else {
                high = mid - 1;
            }
        }
        return -1;
    }
    
    public ListIterator listIterator() {
        return this.items.listIterator();
    }
    
    public ListIterator listIterator(final int index) {
        return this.items.listIterator(index);
    }
    
    public Object remove(final int index) {
        return this.items.remove(index);
    }
    
    public boolean remove(final Object o) {
        return this.items.remove(o);
    }
    
    public boolean removeAll(final Collection c) {
        boolean ret = false;
        final Iterator itr = c.iterator();
        while (itr.hasNext()) {
            if (this.remove(itr.next())) {
                ret = true;
            }
        }
        return ret;
    }
    
    public boolean retainAll(final Collection c) {
        final List list = new SortedList(c.size(), this.comparator);
        list.addAll(c);
        boolean ret = false;
        for (int i = this.size() - 1; i >= 0; --i) {
            if (!list.contains(this.get(i))) {
                this.remove(i);
                ret = true;
            }
        }
        return ret;
    }
    
    public Object set(final int index, final Object element) {
        if (this.compareObjects(this.get(index), element) == 0) {
            return this.items.set(index, element);
        }
        final Object ret = this.items.get(index);
        this.items.remove(index);
        this.add(element);
        return ret;
    }
    
    public int size() {
        return this.items.size();
    }
    
    public List subList(final int fromIndex, final int toIndex) {
        final SortedList result = new SortedList(this.comparator);
        result.items = this.items.subList(fromIndex, toIndex);
        return result;
    }
    
    public Object[] toArray() {
        return this.items.toArray();
    }
    
    public Object[] toArray(final Object[] a) {
        return this.items.toArray(a);
    }
    
    public Comparator getComparator() {
        return this.comparator;
    }
    
    public void setComparator(final Comparator comparator) {
        this.comparator = comparator;
    }
    
    public boolean find(final Object element, final int[] index) {
        int low = 0;
        int high = this.size() - 1;
        while (low <= high) {
            final int mid = (low + high) / 2;
            final int flag = this.compareObjects(element, this.get(mid));
            if (flag == 0) {
                index[0] = mid;
                return true;
            }
            if (flag > 0) {
                low = mid + 1;
            }
            else {
                high = mid - 1;
            }
        }
        index[0] = low;
        return false;
    }
    
    public int find(final Object element) {
        if (this.find(element, this.findStub)) {
            return this.findStub[0];
        }
        return -1;
    }
    
    public void reset(final List items) {
        if (items == null) {
            throw new NullPointerException("items");
        }
        Collections.sort((List<Object>)items, new Comparator() {
            public int compare(final Object o1, final Object o2) {
                return SortedList.this.compareObjects(o1, o2);
            }
        });
        this.items = items;
    }
}
